/* 		REMOTE TEST
 *
 */
#include "softdevice_handler.h"
#include "nrf_drv_gpiote.h"
#include "app_timer.h"
#include "app_uart.h"
#include "nrf_delay.h"

#define RX_PIN_NUMBER 						2
#define TX_PIN_NUMBER 						3
#define BUTTON_L							10
#define BUTTON_R							8
#define NRF_CLOCK_LFCLKSRC      {.source        = NRF_CLOCK_LF_SRC_RC ,  			\
                                 .rc_ctiv       = 16,                    			\
                                 .rc_temp_ctiv  = 0,                     		 	\
                                 .xtal_accuracy = 0}

void UART_init(uint32_t baud_rate);
void gpiote_interrupt_init(uint32_t int_pin);
void gpiote_interrupt_enable(uint32_t int_pin);
void gpiote_interrupt_disable(uint32_t int_pin);
void ble_central_stack_init(void);
void big_function(void);

bool off_flag = false;
int not_used = 0;


int main(void)
{
	UART_init(UART_BAUDRATE_BAUDRATE_Baud115200); printf("\r\n\r\nstart\r\n");
	printf("(A) %d : %d\r\n", nrf_gpio_pin_read(BUTTON_L), nrf_gpio_pin_read(BUTTON_R));
	
    APP_TIMER_INIT(1, 5, false);
	ble_central_stack_init();
	
	gpiote_interrupt_init(BUTTON_L);
	gpiote_interrupt_init(BUTTON_R);
	
    while(1)
    {
		sd_app_evt_wait();
		
		if(not_used)
		{
			big_function();
		}
    
		if(off_flag == true)
		{
			printf("end\r\n");
			nrf_delay_ms(500);
			
			sd_power_system_off();
		}
	}	
	
}



void gpio_event_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
	printf("(%d) %d : %d : ", pin, nrf_gpio_pin_read(BUTTON_L), nrf_gpio_pin_read(BUTTON_R));
	
	if(pin == BUTTON_L)
	{
		gpiote_interrupt_disable(BUTTON_L);
		nrf_delay_ms(500);
		printf("L\n");
		gpiote_interrupt_enable(BUTTON_L);
		off_flag = true;
	}
	else if(pin == BUTTON_R)
	{
		gpiote_interrupt_disable(BUTTON_R);
		nrf_delay_ms(500);
		printf("R\n");
		gpiote_interrupt_enable(BUTTON_R);
		off_flag = true;
	}
}


void gpiote_interrupt_init(uint32_t int_pin)
{
	uint32_t err_code ;
	
	nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_LOTOHI(false);    	//Configure to generate interrupt and wakeup on pin signal low. "false" means that gpiote will use the PORT event, which is low power, i.e. does not add any noticable current consumption (<<1uA). Setting this to "true" will make the gpiote module use GPIOTE->IN events which add ~8uA for nRF52 and ~1mA for nRF51.
	in_config.pull = NRF_GPIO_PIN_PULLDOWN;                                         	//Configure pullup for input pin to prevent it from floting. Pin is pulled down when button is pressed on nRF5x-DK boards, see figure two in http://infocenter.nordicsemi.com/topic/com.nordic.infocenter.nrf52/dita/nrf52/development/dev_kit_v1.1.0/hw_btns_leds.html?cp=2_0_0_1_4
	
	if(!nrf_drv_gpiote_is_init())
	{
		err_code = nrf_drv_gpiote_init();
		APP_ERROR_CHECK(err_code);
	}
	err_code = nrf_drv_gpiote_in_init(int_pin, &in_config, gpio_event_handler);       	//Initialize the wake-up pin
	APP_ERROR_CHECK(err_code);                                                    		//Check error code returned

	gpiote_interrupt_enable(int_pin);
	
}

void gpiote_interrupt_enable(uint32_t int_pin)
{
	nrf_drv_gpiote_in_event_enable(int_pin, true);
}


void gpiote_interrupt_disable(uint32_t int_pin)
{
	nrf_drv_gpiote_in_event_disable(int_pin);
}


void uart_event_handle(app_uart_evt_t * p_event)
{

}

void UART_init(uint32_t baud_rate)
{
    uint32_t                     err_code;

    const app_uart_comm_params_t comm_params =
    {
    	(uint32_t)RX_PIN_NUMBER,
        (uint32_t)TX_PIN_NUMBER,
		(uint32_t)NULL,
        (uint32_t)NULL,
        APP_UART_FLOW_CONTROL_DISABLED,
        false,
        baud_rate
    };

	APP_UART_FIFO_INIT( &comm_params,
                       1,
                       256,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOW,
                       err_code);

    APP_ERROR_CHECK(err_code);
}


void ble_central_stack_init(void)
{
    uint32_t err_code;

    nrf_clock_lf_cfg_t clock_lf_cfg = NRF_CLOCK_LFCLKSRC;
    SOFTDEVICE_HANDLER_INIT(&clock_lf_cfg, NULL);
    ble_enable_params_t ble_enable_params;
    err_code = softdevice_enable_get_default_config(0, 1, &ble_enable_params);
    APP_ERROR_CHECK(err_code);
    CHECK_RAM_START_ADDR(0, 1);

    err_code = softdevice_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);
}


void big_function(void)
{
	// With only one 'printf' line as below the size of th program is small (Program Size: Code=13400 RO-data=308 RW-data=176 ZI-data=4152).
	// Copy-paste the 'printf' line below about 3000 times to get a bigger program (Program Size: Code=91820 RO-data=308 RW-data=176 ZI-data=4152).
	// By changing the code size, the increase of start-up time can be observed (when using bootloader).
	printf("(A) %d : %d\r\n", nrf_gpio_pin_read(BUTTON_L), nrf_gpio_pin_read(BUTTON_R));
}
